/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//AwakeCommonsConfigurator
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 22 sept. 2009 15:03:51 Nicolas de Pomereu
// 21/01/10 19:30 NDP - Add getCallableClassesWithoutAuthentication()
// 26/01/10 19:15 NDP - Add allowFileTransferOperation() & allowSqlOperation
// 28/01/10 16:35 NDP - Add new security rules
// 29/01/10 12:25 NDP - Interface does not reference any more Awake SQL
// 18/11/10 15:10 NDP : Add getEncryptionPasswordForHttpParameters()
// 03/06/11 20:00 NDP : AwakeCommonsConfigurator: Refactor checkLoginAndPassword() to login(), 
//                      getSecretValue() to addSecretForAuthToken() and
//                      getEncryptionPasswordForHttpParameters() to getPasswordForHttpRequestEncryption()
// 08/06/11 12:45 NDP : AwakeCommonsConfigurator: Clean Javadoc
// 09/06/11 16:15 NDP : AwakeCommonsConfigurator: add getBannedUsernames()
// 20/09/11 18:45 NDP : AwakeCommonsConfigurator: getBannedUsernames throws IOException & SQLException
// 20/09/11 18:45 NDP : AwakeCommonsConfigurator: getBannedIPs throws IOException & SQLException
// 20/09/11 18:45 NDP : AwakeCommonsConfigurator: login throws IOException & SQLException
// 20/09/11 18:45 NDP : AwakeCommonsConfigurator: freeConnection throws SQLException
// 20/09/11 19:45 NDP : AwakeCommonsConfigurator: addSecretForAuthToken throws IOException & SQLException
// 14/11/11 16:55 NDP : AwakeCommonsConfigurator: add getLoggingDirectory
// 15/11/11 18:35 NDP : AwakeCommonsConfigurator: add getLogger()
// 21/11/11 13:25 NDP : AwakeCommonsConfigurator: add getConnection(String username, String password)
// 22/11/11 12:35 NDP : AwakeCommonsConfigurator: better Javadoc comments
// 23/11/11 13:30 NDP : AwakeCommonsConfigurator: getPasswordForHttpRequestEncryption()
//			renamed to getEncryptionPassword()
// 05/12/11 19:35 NDP : AwakeCommonsConfigurator: clean javadoc
// 12/12/11 19:20 NDP : AwakeCommonsConfigurator: clean javadoc

package org.awakefw.commons.api.server;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

/**
 * Interface that defines the common User Configuration for the Awake File and
 * Awake SQL Frameworks.
 * <p>
 * All the implemented methods will be called by the Awake Server programs when
 * a client program asks for access to the database from the remote Client side.
 * <p>
 * A concrete implementation should be developed on the Server side by a Awake
 * users in order to:
 * <ul>
 * <li>Define how to extract a JDBC Connection from a Connection Pool.</li>
 * <li>Define how to authenticate the remote (username, password) couple send by
 * the client side.</li>
 * <li>Define if the client must be in secured https.</li>
 * <li>Define the list of banned usernames.</li>
 * <li>Define the list of banned IPs.</li>
 * <li>Define a secret value to reinforce the strength of the hash value used
 * for Authentication Token.</li>
 * <li>Define a password that will be used to encrypt the http request
 * parameters sent by the client side.</li>
 * <li>Define a Logger for Awake internal logging & debug files.</li>
 * </ul>
 * Please note that Awake comes with a Default AwakeCommonsConfigurator
 * implementation that may be extended:
 * <code>DefaultAwakeCommonsConfigurator</code>.
 * 
 * @see DefaultAwakeCommonsConfigurator
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 */
public interface AwakeCommonsConfigurator {
    /**
     * Allows to define if the host must be accessed in secured httpS.
     * 
     * If true, the Awake File Manager or Awake SQL Manager (AwakeFileManager or AwakeSqlManager Servlet) will refuse
     * access to client program calling the Servlet in not secured http.
     * 
     * @return <code>true</code> if the host must be in httpS
     */
    public boolean forceSecureHttp();

    /**
     * Allows to define the set of banned usernames. The Awake Server will
     * refuse access to client programs calling with a username in the set.
     * <p>
     * 
     * @return the set of banned usernames that are not allowed to access the
     *         service
     * @throws IOException
     *             if an IOException occurs
     * @throws SQLException
     *             if a SQLException occurs
     */
    public Set<String> getBannedUsernames() throws IOException, SQLException;

    /**
     * Allows to define the list of banned IP addresses. The Awake Server will
     * refuse access to client programs calling with an address in the list.
     * <p>
     * Subnet notations are supported: 1.1.1.1/255.255.255.255 or 1.1.1.1/32
     * (CIDR-Notation).
     * <p>
     * 
     * @return the list of banned IP addresses, that are not allowed to access
     *         the service
     * @throws IOException
     *             if an IOException occurs
     * @throws SQLException
     *             if a SQLException occurs
     */
    public List<String> getBannedIPs() throws IOException, SQLException;

    /**
     * Allows to authenticate the remote (username, password) couple send by the
     * client side and to give access to the Awaker Server.
     * <p>
     * The Awake Server will call the method in order to grant or not client 
     * access. 
     * <p>
     * Typical usage would be to check the (username, password) couple against a
     * table in a SQL database or against a LDAP, etc.
     * 
     * @param username
     *            the username sent by the client login
     * @param password
     *            the password to connect to the server
     *            <p>
     * @return <code>true</code> if the (login, password) couple is
     *         correct/valid. If false, the client side will not be authorized
     *         to send any command.
     * @throws IOException
     *             if an IOException occurs
     * @throws SQLException
     *             if a SQLException occurs
     */
    public boolean login(String username, char[] password) throws IOException,
	    SQLException;

    /**
     * <p>
     * Attempts to establish a connection with an underlying data source.
     * 
     * @return a connection to the data source
     * @exception SQLException
     *                if a database access error occurs
     */
    public Connection getConnection() throws SQLException;

    /**
     * <p>
     * Attempts to establish a connection with an underlying data source.
     * 
     * @param username
     *            the database user on whose behalf the connection is being made
     * @param password
     *            the user's password
     * @return a connection to the data source
     * @exception SQLException
     *                if a database access error occurs
     */
    public Connection getConnection(String username, String password)
	    throws SQLException;

    /**
     * Allows to define a secret value that will secure the authentication. <br>
     * After login() succeeds, Awake builds an Authentication Token that will be
     * use by each following client call in order to authenticate the calls. <br>
     * <br>
     * The Authentication Token default value built by Awake File and Awake SQL is: <br>
     * <code>SHA-1(username + hostname)</code> first 20 hexadecimal characters.<br>
     * where:
     * <ul>
     * <li>username: the username of the client.</li>
     * <li>hostname: the hostname returned by the Linux/Window shell command
     * "hostname" on the server side.</li>
     * </ul>
     * If you want to reinforce the Authentication Token with a secret value so
     * that no one can guess or forge a false Authentication Token, use this
     * method to return a fixed secret value. <br>
     * <br>
     * The Awake Server will use this value to reinforce the strength of the
     * hash value used for Authentication Token at each method call. 
     * <br>The  Authentication Token will then be:<br>
     * <code>SHA-1(username + hostname + secretValue)</code> first 20 hexadecimal characters.<br>
     * where:
     * <ul>
     * <li>secretValue: secret fixed value returned by addSecretForAuthToken.</li>
     * </ul>
     * 
     * @return the secret value to add to the strength of the hash used as
     *         Authentication Token
     * @throws IOException
     *             if an IOException occurs
     * @throws SQLException
     *             if a SQLException occurs
     */
    public String addSecretForAuthToken() throws IOException, SQLException;

    /**
     * Allows to define the password that is used to encrypt from the Client all the
     * request parameters values for security reason (obsfucation and transport
     * encryption).
     * 
     * @return the password used to encrypt from Client all the request parameters
     *         values for security reason.
     */
    public char[] getEncryptionPassword();

    /**
     * Returns the {@link Logger} that will be used by Awake File or Awake SQL
     * logging & debug files:
     * <ul>
     * <li>Most internal Awake log() calls are done with 
     * Level.WARNING.</li>
     * <li>Exceptions thrown are logged with Level.SEVERE.</li>
     * </ul> 
     * It is not necessary nor recommended to implement this method; do it only
     * if you want take control of the logging to modify the default
     * characteristics of {@link DefaultAwakeCommonsConfigurator#getLogger()}.
     * 
     * @return the java.util.logging.Logger that will be used by Awake File or Awake SQL
     *         logging & debug files.
     */
    public Logger getLogger() throws IOException;

}
