/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//Sha1.java
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 14 oct. 2005 17:01:13 Nicolas de Pomereu
// 31/12/05 13:10 NDP - fix bug on order parameter in getInstance
// 06/01/06 17:30 NDP - add File Hash
// 17/09/08 10:45 NDP - Silent close of input stream in finally

package org.awakefw.file.api.util;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.io.IOUtils;

/**
 * Hash funtions with only SUN provider and SHA-1
 * 
 * @author Nicolas de Pomereu
 * 
 */
public class Sha1 {

    // default read buffer size
    public static final int READ_BUFFER_SIZE = 4096;

    // Default Constructor
    public Sha1() {

    }

    /**
     * 
     * Computes the SHA-1 Hash code of a byte array as an hexa String <br>
     * 
     * @param bIn
     *            the bytes to hash <br>
     * @return the hash value
     * @exception NoSuchAlgorithmException
     *                if the algorithm is not available from the provide
     * @exception NoSuchProviderException
     *                if the provider is not available in the environment
     */

    public byte[] getHash(byte bIn[]) throws NoSuchAlgorithmException,
	    NoSuchProviderException {
	MessageDigest mdInstance = MessageDigest.getInstance("SHA-1", "SUN");

	// Bytes containing the MD5 or SHA-1 Hash Code
	byte[] bHash = new byte[160];

	mdInstance.update(bIn);

	bHash = mdInstance.digest();

	return bHash;
    }

    /**
     * 
     * Computes the Hash code of a byte array as an hexa String. <br>
     * 
     * @param bIn
     *            the bytes to hash <br>
     * @return the hash value in Hexa String
     * @exception NoSuchAlgorithmException
     *                if the algorithm is not available from the provider
     * @exception NoSuchProviderException
     *                if the provider is not available in the environment
     */

    public String getHexHash(byte bIn[]) throws NoSuchAlgorithmException,
	    NoSuchProviderException {
	// Codec 1.4:
	// return Hex.encodeHexString(getHash(bIn));

	// Codec 1.3:
	return new String(Hex.encodeHex(getHash(bIn)));
    }

    /**
     * Gets the fileIn hash.
     * 
     * @param sFilePath
     *            the fileIn path
     * @return the fileIn hash as bytes
     * 
     * @exception IOException
     *                an I/O error occured
     * @exception NoSuchAlgorithmException
     *                hash algorithm not found
     * @exception NoSuchProviderException
     *                hash provider not found
     */

    public byte[] getFileHash(String sFilePath) throws IOException,
	    NoSuchAlgorithmException, NoSuchProviderException {
	File fTmp = new File(sFilePath);

	FileInputStream fisIn = new FileInputStream(fTmp);
	BufferedInputStream bisIn = new BufferedInputStream(fisIn);

	DataInputStream disIn = null;

	try {
	    disIn = new DataInputStream(bisIn);

	    // Bytes containing the MD5 or SHA-1 Hash Code
	    byte[] bHash = new byte[160];

	    MessageDigest mdInstance = MessageDigest
		    .getInstance("SHA-1", "SUN");

	    int nBytesRead;
	    int nTotal = disIn.available();

	    if (nTotal == 0) {
		return null;
	    }

	    if (nTotal > READ_BUFFER_SIZE) {
		nTotal = READ_BUFFER_SIZE;
	    }

	    byte bRead[] = new byte[nTotal];

	    while ((nBytesRead = disIn.read(bRead)) > 0) {
		mdInstance.update(bRead, 0, nBytesRead);
	    }

	    bHash = mdInstance.digest();

	    return bHash;
	} finally {
	    IOUtils.closeQuietly(disIn);
	}

    }

    /**
     * Gets the fileIn hash as an hexa string value.
     * 
     * @param sFilePath
     *            the fileIn path
     * @return the fileIn hash as an hexa string
     * 
     * @exception IOException
     *                an I/O error occured
     * @exception NoSuchAlgorithmException
     *                hash algorithm not found
     * @exception NoSuchProviderException
     *                hash provider not found
     */

    public String getHexFileHash(String sFilePath) throws IOException,
	    NoSuchAlgorithmException, NoSuchProviderException {
	String sHashResult = new String();

	byte[] bHash = this.getFileHash(sFilePath);

	if (bHash == null) {
	    return "NULL"; // Fow Lastwall compliance
	}

	// Codec 1.4:
	// sHashResult = Hex.encodeHexString(bHash);

	// Codec 1.3:
	sHashResult = new String(Hex.encodeHex(bHash));
	return sHashResult;
    }

}
