/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//FileUtil.java
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 26 mai 2006 19:36:53 Nicolas de Pomereu
// 01/03/07 11:35 NDP - Add isUnlocked()
// 13/04/07 15:55 NDP - Modify get/Put text Content to handle read texts
// 30/04/07 15:55 NDP - New isUnlockedForRead() and new name for isUnlocked() : isUnlockedForWrite()
// 31/04/07 20:55 NDP - Directories are always readable & writable
// 01/04/10 11:10 NDP - getUniqueId() moved to FileUtil
// 14/06/10 11:40 NDP - getUniqueId(): use now an random because two thread can do access at same time
// 12/11/11 18:05 NDP : FileUtil: getUniqueId() uses now "-" as separator char
// 12/11/11 20:20 NDP : FileUtil: getUniqueId(): rewritten to be *sure* to have a unique ID: 
//			using: UUID idOne = UUID.randomUUID();
// 30/11/11 18:50 NDP : FileUtil: add cleanTempDir() - not used

package org.awakefw.file.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.Vector;


/**
 * Misc file utilities
 * 
 * @author Nicolas de Pomereu
 * 
 */
public class FileUtil {

    /** The DEBUG flag */
    public boolean DEBUG = false;

    public static String CR_LF = System.getProperty("line.separator");

    /** List of files */
    private List<File> m_fileList = new Vector<File>();

    /**
     * Constructor
     */
    public FileUtil() {
	super();
    }

    /**
     * Return true if the filename is a Linux possible Filename
     * 
     * @param filename
     *            the filename to test
     * @return true if the filename is a Linux Filename
     */
    public static boolean isPossibleLinuxFilename(String filename) {
	if (filename.indexOf("\\") != -1 || // Windows
		filename.indexOf("/") != -1 || // Windows
		filename.indexOf(":") != -1 || // Windows
		filename.indexOf("*") != -1 || // Windows
		filename.indexOf("?") != -1 || // Windows
		filename.indexOf("\"") != -1 || // Windows
		filename.indexOf("<") != -1 || // Windows
		filename.indexOf(">") != -1 || // Windows
		filename.indexOf("|") != -1 || // Windows
		filename.indexOf("@") != -1 || // Linux
		filename.indexOf(" ") != -1) // Linux
	{
	    return false;
	} else {
	    return true;
	}
    }

    /**
     * Return true if the fils exists, is readable and is not locked by another
     * process <br>
     * New version that works because it uses RandomAccessFile and will throw an
     * exception if another file has locked the file
     */
    public boolean isUnlockedForRead(File file) {
	if (file.exists() && file.canRead()) {
	    if (file.isDirectory()) {
		return true;
	    }

	    try {
		RandomAccessFile raf = new RandomAccessFile(file, "r");

		raf.close();

		debug(new Date() + " " + file + " OK!");

		return true;
	    } catch (Exception e) {
		debug(new Date() + " " + file + " LOCKED! " + e.getMessage());
	    }
	} else {
	    debug(new Date() + " " + file + " LOCKED! File exists(): "
		    + file.exists() + " File canWrite: " + file.canWrite());
	}

	return false;

    }

    /**
     * Return true if the fils exists and is readable and writable not locked by
     * another process <br>
     * New version that works because it uses RandomAccessFile and will throw an
     * exception if another file has locked the file
     */
    public boolean isUnlockedForWrite(File file) {
	if (file.exists() && file.canWrite()) {
	    if (file.isDirectory()) {
		return true;
	    }

	    try {
		RandomAccessFile raf = new RandomAccessFile(file, "rw");

		raf.close();

		debug(new Date() + " " + file + " OK!");

		return true;
	    } catch (Exception e) {
		debug(new Date() + " " + file + " LOCKED! " + e.getMessage());
	    }
	} else {
	    debug(new Date() + " " + file + " LOCKED! File exists(): "
		    + file.exists() + " File canWrite: " + file.canWrite());
	}

	return false;

    }

    /**
     * Recuse research of a files. Result is stored in m_fileList
     * 
     * @param fileList
     *            the initial list
     * @param recurse
     *            if true, search inside directories
     */

    private void searchFiles(File[] fileList, boolean recurse) {
	if (fileList == null) {
	    throw new IllegalArgumentException("File list can't be null!");
	}

	for (int i = 0; i < fileList.length; i++) {
	    m_fileList.add(fileList[i]);

	    if (fileList[i].isDirectory()) {
		if (recurse) {
		    searchFiles(fileList[i].listFiles(), recurse);
		}
	    }
	}
    }

    /**
     * Extract all files from a list of files, with recurse options
     * 
     * @param fileList
     *            the initial list
     * @param recurse
     *            if true, search inside directories
     * 
     * @return the extracted all files
     */
    public List<File> listAllFiles(List<File> fileList, boolean recurse) {

	if (fileList == null) {
	    throw new IllegalArgumentException("File list can't be null!");
	}

	if (fileList.isEmpty()) {
	    return new Vector<File>();
	}

	File[] files = new File[fileList.size()];

	for (int i = 0; i < fileList.size(); i++) {
	    files[i] = fileList.get(i);
	}

	searchFiles(files, recurse);
	return m_fileList;
    }

    /**
     * 
     * Extract the files that are not directories from a file list
     * 
     * @param fileList
     *            the initial list (firt level only)
     * @return the file list
     */
    public List<File> extractFilesOnly(List<File> fileList) {
	if (fileList == null) {
	    throw new IllegalArgumentException("File list can't be null!");
	}

	if (fileList.isEmpty()) {
	    return new Vector<File>();
	}

	List<File> files = new Vector<File>();

	for (int i = 0; i < fileList.size(); i++) {
	    if (!fileList.get(i).isDirectory()) {
		files.add(fileList.get(i));
	    }
	}

	return files;

    }

    /**
     * 
     * Extract the directories only from a file list
     * 
     * @param fileList
     *            the initial list (firt level only)
     * @return the file list
     */
    public List<File> extractDirectoriesOnly(List<File> fileList) {
	List<File> directories = new Vector<File>();

	for (int i = 0; i < fileList.size(); i++) {
	    if (fileList.get(i).isDirectory()) {
		directories.add(fileList.get(i));
	    }
	}

	return directories;

    }

    /**
     * Put the content of a file as HTML into a String <br>
     * No carriage returns will be included in output String
     * 
     * @param fileIn
     *            The HTML text file
     * @return The content in text
     * @throws IOException
     */
    public String getHtmlContent(File fileIn) throws IOException {
	if (fileIn == null) {
	    throw new IllegalArgumentException("File name can't be null!");
	}

	String text = "";

	BufferedReader br = new BufferedReader(new FileReader(fileIn));

	String line = null;

	while ((line = br.readLine()) != null) {
	    text += line;
	}

	return text;
    }

    /**
     * debug tool
     */
    private void debug(String s) {
	if (DEBUG) {
	    AwakeLogger.log(s);
	}
    }

    /**
     * Build a unique string
     * 
     * @return a unique string
     */
    public static synchronized String getUniqueId() {
//	long uniqueLong = new Date().getTime();
//
//	String unique = Long.toString(uniqueLong);
//
//	double randomDouble = Math.random();
//	String random = Double.toString(randomDouble);
//	random = StringUtils.substringAfter(random, ".");
//	random = StringUtils.substring(random, 0, 4);
//
//	unique += "-" + random;

	UUID idOne = UUID.randomUUID();
	
	return idOne.toString();
    }

    /**
     * @return the Awake temp directory (create it if not exists)
     */
    public static String getAwakeTempDir() {
	String dir = System.getProperty("java.io.tmpdir");
	if (!dir.endsWith(File.separator)) {
	    dir += File.separator;
	}
	String tempDir = dir + ".awake" + File.separator;

	File tempDirFile = new File(tempDir);
	tempDirFile.mkdir();
	return tempDir;
    }
    
//    /**
//     * Cleans the temp directory
//     */
//    public static void cleanTempDir()
//    {
//	try {
//	    String dir = getAwakeTempDir();
//	    File [] files = new File(dir).listFiles();
//	    if (files != null)
//	    {
//		for (int i = 0; i < files.length; i++) {
//		    boolean deleted = files[i].delete();
//		    if (! deleted) {
//			AwakeLogger.log("Could not delete in Awake Temp Dir file: " + files[i].getName());
//		    }
//		}
//	    }
//	} catch (Exception e) {
//	    AwakeLogger.log(e.toString());
//	}	
//    }

}
